// Copyright 2001  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef OLENA_MORPHO_EROSION_HH
# define OLENA_MORPHO_EROSION_HH

# include "basics.hh"
# include "morpho/stat.hh"
# include "morpho/fast_morpho.hh"

namespace oln {

  namespace morpho {

    template<class _I, class _E> inline
    Concrete(_I) erosion(const image<_I>& _input, const struct_elt<_E>& _se)
    {
      Exact_cref(I, input);
      Exact_cref(E, se);

      Concrete(I) output(input.size());
      border::adapt_copy(input, se.delta());
      Iter(I) p(input);

      for_all (p)
	output[p] = morpho::min(input, p, se);
      return output;
    }

    template<class _I, class _E> inline
    Concrete(_I) n_erosion(const image<_I> & input,
			const struct_elt<_E>& se,
			unsigned n)
    {
      // FIXME: We should do this by distance thresolding
      precondition(n > 0);
      Concrete(_I) output = input.clone();
      for (unsigned i = 0; i < n; ++i)
	{
	  Concrete(_I) work = erosion(output, se);
	  output = work;
 	}
      return output;
    }

    namespace fast {
      template<class I, class E> inline
      Concrete(I) erosion(const image<I>& input, const struct_elt<E>& se)
      {
        return fast_morpho<I, E, utils::min>(input, se);
      }
    }

  } // end of morpho

} // end of oln


#endif // ! OLENA_MORPHO_EROSION_HH
